/*
 DSPRobotics
 PIC Clicker Relay Click - HID  V1.0
 Language: Mikro C Pro
 Chip: P18F47J53
 CLK: 48Mhz
 
 HID Data:
 Char 0 Read:
 bit 0 = LED1
 bit 1 = LED2
 Bit 2 = Relay1
 Bit 3 = Relay2
 
 Char 0 Write:
 bit 0 = Button1
 bit 1 = Button2

 Aplication: Ideal for using with the FlowStone Graphical Programming Language.
 www.dsprobotics.com

 */
/*---------------------------------------------------------------*/
// USB buffers
unsigned char readbuff[64] absolute 0x500;   // Buffers should be in USB RAM, please consult datasheet
unsigned char writebuff[64] absolute 0x540;
//General
char kk;
char i;
// pin definitions
//LEDs
sbit LD1 at LATA0_bit;    //LED 1
sbit LD2 at LATA1_bit;    //LED 2
sbit LD1_Direction at TRISA0_bit;
sbit LD2_Direction at TRISA1_bit;
//Button Inputs
sbit T1 at RD3_bit;   //Button 1
sbit T2 at RD2_bit;   //Button 2
sbit T1_Direction at TRISD3_bit;
sbit T2_Direction at TRISD2_bit;
//Click Relay
sbit Relay1 at LATC6_bit; // pin
sbit Relay1_Direction at TRISC6_bit;    //Direction
sbit Relay2 at LATB3_bit; // pin
sbit Relay2_Direction at TRISB3_bit;    //Direction
/*---------------------------------------------------------------*/
// Interrupt routine
void interrupt() {

 USB_Interrupt_Proc();                   // USB servicing is done inside the interrupt
}
/*---------------------------------------------------------------*/
void clear_USB_buffers()
{
   for (i=0; i<63;i++)
   {
   readbuff[i] = 0;
   writebuff[i] = 0;
   }
}
/*---------------------------------------------------------------*/
char read_button1() {
  if (Button(&PORTD, 3, 2, 0)) {               // Detect logical one
      return  1;                              // Update flag
    } else return 0;
}
/*---------------------------------------------------------------*/
char read_button2() {
  if (Button(&PORTD, 2, 2, 0)) {               // Detect logical one
      return  2;                              // Update flag
    } else return 0;
}
/*---------------------------------------------------------------*/
void main(){
  long cnt = 0;

  ANCON0 = 0xFF;               // Default all pins to digital
  ANCON1 = 0xFF;               //
  TRISD = 0;
  TRISB = 0;
  TRISA = 0;
  RCSTA1 = 1;
  //Buttons
  T1_Direction = 1;         // Set direction for buttons
  T2_Direction = 1;
  //Leds
  LD1_Direction = 0;        // Set direction for LEDS
  LD2_Direction = 0;
  LD1 = 0;
  LD2 = 0;                 // turn off leds
  //Click Relay
  Relay1_Direction = 0;     //Set Direction for Relay Pins
  Relay2_Direction = 0;
  Relay1 = 0;
  Relay2 = 0;
 //HID
  clear_USB_buffers();
  HID_Enable(&readbuff,&writebuff);       // Enable HID communication
 /*---------------------------------------------------------------*/
 //Main Loop
  while (1){
  //USB
  kk = HID_Read();
  if (kk != 0)
  {
  //LEDS
   if ((readbuff[0] & 0x01 )> 0) {
     LD1 = 1;
   }  else LD1 = 0;
   if ((readbuff[0] & 0x02 )> 0) {
     LD2 = 1;
   }  else LD2 = 0;
   //Buttons
   writebuff[0]=0;
   writebuff[0] = writebuff[0] | read_button1() ;
   writebuff[0] = writebuff[0] | read_button2() ;
   //Relay Click
   if ((readbuff[0] & 0x04 )> 0) {
     Relay1 = 1;
   }  else Relay1 = 0;
   if ((readbuff[0] & 0x08 )> 0) {
     Relay2 = 1;
   }  else Relay2 = 0;
   //HID
   HID_Write(&writebuff,64);
  }
 }
}